# HTTP Server

## Overview
The base adapter image uses Debian as its base OS.

The server inside the base adapter image was generated by using:
- [OpenAPI-Spec](https://github.com/swagger-api/swagger-core/wiki) to define the API.
- [swagger-codegen](https://github.com/swagger-api/swagger-codegen) to generate serrver stubs using the file defined with OpenAPI.
- [Connexion framework](https://github.com/zalando/connexion) to map each of the endpoint specified in the OpenAPI file to map to our Python functions.

## Requirements
Python 3.9+

## Regenerate the server
To regenerate the server: 
1.  If necessary, update the swagger API JSON file(s) in `vmware_aria_operations_integration_sdk/api`
2.  Run the following command from the `images` directory:
    ```shell
    python3 update-base-python-adapter-server.py
    ```
    This script will combine both of the JSON files inside the 
    `vmware_aria_operations_integration_sdk/api` and then generate a JSON file at 
    `images/base-python-adapter/combined.json`. The script then runs a swagger container 
    image that reads the combined JSON file, generates an HTTP server that implements
    the API specification, and writes the code to the `base-python-adapter` directory.
3.  Fix any conflicts that arise from the new server and the existing one. In particular:
    *  `images/base-python-adapter/swagger_server/swagger/swagger.yaml` will need to be
        updates to the `x-openapi-router-controller` for each path. The implementation 
        for the api paths are in `images/base-python-adapter/swagger_server/controllers/controller.py`,
        and as such the `x-openapi-router-controller` should be set to 
       `swagger_server.controllers.controller`.
4.  Any updates to the API that require changes in behavior or new behavior should be
    implemented in the controller.

## Testing the server using `mp-test`
In many cases it is useful to test changes to the server using a test adapter and 
`mp-test`. To do this follow these steps:
1.  Create a new management pack project using `mp-init`.
2.  Copy the `base-python-adapter` directory into the root project directory.
3.  Modify the Dockerfile as follows:
    ```dockerfile
    FROM python:3.11.1-slim-bullseye

    RUN mkdir -p /var/log/
    RUN chmod 777 /var/log/

    RUN useradd --create-home --shell /bin/bash vrops-adapter-user
    USER vrops-adapter-user
    WORKDIR /home/vrops-adapter-user/src/app

    COPY base-python-adapter .
    RUN pip3 install --no-cache-dir -r requirements.txt

    # Start Original Adapter Dockerfile
    COPY adapter_requirements.txt .
    RUN pip3 install -r adapter_requirements.txt --upgrade
    COPY commands.cfg .
    COPY app app
    # End Original Adapter Dockerfile

    EXPOSE 8080
    EXPOSE 443

    ENTRYPOINT ["python3"]
    CMD ["-m", "swagger_server"]
    ```
4.  The new server and/or base image can now be tested using `mp-test`. Changes in 
    the project's `base-python-adapter` will take effect the next time `mp-test` is
    run. If changes are make, they will need to be manually copied back to the SDK
    project.

## Testing the server locally
To run the server outside a container/adapter, run the following from the root directory:

```
pip3 install -r requirements.txt
python3 -m swagger_server
```

and open your browser to here:

```
http://localhost:8080/ui/
```

Your Swagger definition lives here:

```
http://localhost:8080/swagger.json
```

To launch the integration tests, use tox:
```
sudo pip install tox
tox
```